#include "pgjson/pgjson.hpp"
#include "test/Expectation.h"

#include <stdexcept>


namespace xs { namespace test
{
    Expectation::Expectation(const pgjson::CValue& object)
    : name_(object["command"].as<std::string>())
    {
        request_ = object["param"];

        if (!object.hasMember("expect")) {
            return;
        }

        const auto& expect = object["expect"];
        response_ = expect["response"];

        if (expect.hasMember("runs")) {
            for (const auto& element : expect["runs"]) {
                if (!element.isString()) {
                    throw std::runtime_error("element in \"runs\" is not string");
                }
                runs_.emplace(element.as<std::string>());
            }
        }

        if (expect.hasMember("posts")) {
            for (const auto& element : expect["posts"]) {
                if (!element.isString()) {
                    throw std::runtime_error("element in \"posts\" is not string");
                }
                posts_.emplace(element.as<std::string>());
            }
        }
    }

    bool Expectation::verify(const pgjson::CValue& object) noexcept
    {
        if (!object.isObject()) {
            return false;
        }

        if (!(object.hasMember("command") && object["command"].isString())) {
            return false;
        }

        if (!object.hasMember("param")) {
            return false;
        }

        if (!(object["param"].isString() || object["param"].isObject())) {
            return false;
        }

        if (!object.hasMember("expect")) {
            return true;
        }

        if (!object["expect"].isObject()) {
            return false;
        }

        const auto& expect = object["expect"];
        if (!expect.hasMember("response")) {
            return false;
        }

        if (!(expect["response"].isString() || expect["response"].isObject())) {
            return false;
        }

        if (expect.hasMember("runs") && !expect["runs"].isArray()) {
            return false;
        }

        if (expect.hasMember("posts") && !expect["posts"].isArray()) {
            return false;
        }

        return true;
    }
} }
