#include "pgjson/pgjson.hpp"
#include "test/Service.h"

#include <memory>


namespace xs { namespace test 
{
    Service::Service(const pgjson::CValue& object)
    : Service(
        object[object.hasMember("command") ? "command" : "event"].as<std::string>()
    )
    {
        if (object.hasMember("command")) {
            response_ = object["response"];

            if (object.hasMember("param")) {
                request_ = object["param"];
            }
        }
    }

    bool Service::verify(const pgjson::CValue& object) noexcept
    {
        if (!object.isObject()) {
            return false;
        }

        static const auto isStringOrObject = [](const pgjson::CValue& object) {
            return object.isString() || object.isObject();
        };
        if (object.hasMember("command")) {
            if (object.hasMember("param")) {
                if (!isStringOrObject(object["param"])) {
                    return false;
                }
            }

            return object["command"].isString()
                && isStringOrObject(object["response"])
                ;
        } else if (object.hasMember("event")) {
            return object["event"].isString();
        }

        return false;
    }

    Service::Service(std::string name) noexcept
    : name_(std::move(name))
    , response_(boost::none)
    { }

    bool Service::isCommand() const noexcept
    {
        return static_cast<bool>(response_);
    }

    bool Service::isEvent() const noexcept
    {
        return !isCommand();
    }
} }
